/* Copyright (c) 2009-2010 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an
 * express license agreement from NVIDIA Corporation is strictly prohibited.
 */

/**
 * @file
 * <b>NVIDIA Tegra: OpenMAX Camera Extension Interface</b>
 *
 */

/**
 * @defgroup nv_omx_il_camera Camera
 *
 * This is the NVIDIA OpenMAX camera class extension interface.
 *
 * These extensions include auto-focus, auto-exposure, auto-whitebalance, half-press,
 * focus regions, sharpness, hue, framerate, encoder control, edge enhancement and more.
 *
 * @ingroup nvomx_camera_extension
 * @{
 */

#ifndef NVOMX_CameraExtensions_h_
#define NVOMX_CameraExtensions_h_

typedef float NVX_F32;

/** Holds a floating point rectangle. */
typedef struct NVX_RectF32
{
    OMX_S32 left;
    OMX_S32 top;
    OMX_S32 right;
    OMX_S32 bottom;
} NVX_RectF32;

#define NVX_MAX_FOCUS_REGIONS  8
#define NVX_MAX_DIRECT_FOCUSER_CONTROL_LENGTH  16
#define NVX_VIDEOENC_DCI_SIZE 80
#define MAX_NUM_SENSOR_MODES  8
#define NVX_MAX_CAMERACONFIGURATION_LENGTH 64

typedef enum  NVX_IMAGE_FLASHCONTROLTYPE {
   NVX_IMAGE_FlashControlKhronosExtensions = OMX_IMAGE_FlashControlKhronosExtensions,
   NVX_IMAGE_FlashControlConvergeAssistPlusStrobe,
   NVX_IMAGE_FlashControlMax = 0x7FFFFFFF
}  NVX_IMAGE_FLASHCONTROLTYPE;

typedef enum NVX_VIDEO_ERROR_RESILIENCY_LEVEL_TYPE {
    NVX_VIDEO_ErrorResiliency_None = 0,
    NVX_VIDEO_ErrorResiliency_Low,
    NVX_VIDEO_ErrorResiliency_High,
    NVX_VIDEO_ErrorResiliency_Invalid = 0x7FFFFFFF
} NVX_VIDEO_ERROR_RESILIENCY_LEVEL_TYPE;

typedef enum NVX_VIDEO_APPLICATION_TYPE {
    NVX_VIDEO_Application_Camcorder = 0,    /**< Timestamps set for camcorder */
    NVX_VIDEO_Application_VideoTelephony,   /**< Timestamps set for telephony */
    NVX_VIDEO_Application_Invalid = 0x7FFFFFFF
} NVX_VIDEO_APPLICATION_TYPE;

typedef enum NVX_VIDEO_RATECONTROL_MODE{
    NVX_VIDEO_RateControlMode_CBR = 0,
    NVX_VIDEO_RateControlMode_VBR,
    NVX_VIDEO_RateControlMode_Invalid = 0x7FFFFFFF
}NVX_VIDEO_RATECONTROL_MODE;

/** Param extension index to fine tune video encoder configuration.
 *  See ::NVX_PARAM_VIDENCPROPERTY
 */
#define NVX_INDEX_PARAM_VIDEO_ENCODE_PROPERTY "OMX.Nvidia.index.param.video.encode.prop"
/** Holds data to fine tune video encoder configuration. */
typedef struct NVX_PARAM_VIDENCPROPERTY
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */

    NVX_VIDEO_APPLICATION_TYPE eApplicationType;                    /**< Application Type */
    NVX_VIDEO_ERROR_RESILIENCY_LEVEL_TYPE eErrorResiliencyLevel;    /**< Error Resiliency Level */
    OMX_BOOL bSvcEncodeEnable;                                      /**< Boolean to enable H.264 Scalable Video Codec mode */
} NVX_PARAM_VIDENCPROPERTY;
/** Param extension index to set video encoder rate control mode.
 *  See ::NVX_PARAM_RATECONTROLMODE
 */
 #define NVX_INDEX_PARAM_RATECONTROLMODE "OMX.Nvidia.index.param.ratecontrolmode"
/** Holds data to set video encoder rate control mode.
 * See ::NVX_IndexParamRateControlMode
 */
typedef struct NVX_PARAM_RATECONTROLMODE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */

    NVX_VIDEO_RATECONTROL_MODE eRateCtrlMode;
}NVX_PARAM_RATECONTROLMODE;

/** Param extension index to configure the video encode quantization range.
 *  See ::NVX_CONFIG_VIDENC_QUANTIZATION_RANGE
 */
#define NVX_INDEX_CONFIG_VIDEO_ENCODE_QUANTIZATION_RANGE "OMX.Nvidia.index.config.video.encode.quantizationrange"
/**
 * Holds information for configuring video compression quantization parameter limits
 * parameter values. Codecs may support different minimum/maximum QP values for different
 * frame types. The default value used by all \a MinQp and \a MaxQp members should be set to an
 * invalid value, so that the encoder rate control parameter will not be affected. If control
 * is required for \a MinQp, \a MaxQp, or both, then set a valid value for that parameter.
 */
typedef struct NVX_CONFIG_VIDENC_QUANTIZATION_RANGE {
    OMX_U32 nSize;                /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;     /**< OMX specification version info */
    OMX_U32 nPortIndex;           /**< Port that this structure applies to */
    OMX_U32 nMinQpI;              /**< Minimum QP value to use for index frames */
    OMX_U32 nMaxQpI;              /**< Maximum QP value to use for index frames */
    OMX_U32 nMinQpP;              /**< Minimum QP value to use for P frames */
    OMX_U32 nMaxQpP;              /**< Maximum QP value to use for P frames */
    OMX_U32 nMinQpB;              /**< Minimum QP value to use for B frames */
    OMX_U32 nMaxQpB;              /**< Maximum QP value to use for B frames */
} NVX_CONFIG_VIDENC_QUANTIZATION_RANGE;


/** Param extension index to enable/disable stringent bitrate for encoder.
 *  See ::OMX_CONFIG_BOOLEANTYPE
 */
#define NVX_INDEX_PARAM_VIDEO_ENCODE_STRINGENTBITRATE "OMX.Nvidia.index.param.video.encode.stringentbitrate"

/**
 * Config extension index to set/get stereo metadata.
 */
#define NVX_INDEX_CONFIG_VIDEO_STEREOINFO "OMX.Nvidia.index.config.videostereoinfo"

// TODO: Need to finalize on the exact defines to expose to the app
// Frame Packing Format
typedef enum _NVX_VIDEO_FRAMEPACK_TYPE {
    NVX_VIDEO_FRAMEPACK_Type_Checker  = 0, // Checkerboard
    NVX_VIDEO_FRAMEPACK_Type_ColInt   = 1, // Column Interleaved
    NVX_VIDEO_FRAMEPACK_Type_RowInt   = 2, // Row Interleaved
    NVX_VIDEO_FRAMEPACK_Type_SbS      = 3, // Side by Side
    NVX_VIDEO_FRAMEPACK_Type_TB       = 4, // Top Bottom
    NVX_VIDEO_FRAMEPACK_Type_FrameInt = 5, // Temporal Frame Interleaved
    NVX_VIDEO_FRAMEPACK_TypeInvalid   = 0x7FFFFFFF
} NVX_VIDEO_FRAMEPACK_TYPE;

// Content Interpretation Type
typedef enum _NVX_VIDEO_CONTENT_TYPE {
    NVX_VIDEO_CONTENT_Type_Undef    = 0, // Undefined
    NVX_VIDEO_CONTENT_Type_LR       = 1, // Left first
    NVX_VIDEO_CONTENT_Type_RL       = 2, // Right first
    NVX_VIDEO_CONTENT_TypeInvalid   = 0x7FFFFFFF
} NVX_VIDEO_CONTENT_TYPE;

// Stereo Metadata structure
typedef struct NVX_CONFIG_VIDEO_STEREOINFO
{
    OMX_U32 nSize;                    /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;         /**< NVX extensions specification version */
    OMX_U32 nPortIndex;               /**< Port that this struct applies to */

    OMX_U32 stereoEnable;             /**< Stereo Enable / Disable for this buffer */
    NVX_VIDEO_FRAMEPACK_TYPE fpType;  /**< Stereo Frame-packing type */
    NVX_VIDEO_CONTENT_TYPE contentType; /**< Stereo Content Type */
} NVX_CONFIG_VIDEO_STEREOINFO;

/** Config extension index to enable camera test pattern.
 *  See ::NVX_CONFIG_CAMERATESTPATTERN
 */
#define NVX_INDEX_CONFIG_CAMERATESTPATTERN "OMX.Nvidia.index.config.cameratestpattern"
/** Holds data to enable camera test pattern. */
typedef struct NVX_CONFIG_CAMERATESTPATTERN
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32 TestPatternIndex;       /**< Boolean to enable test pattern */
}NVX_CONFIG_CAMERATESTPATTERN;

/** Config extension index to set duration in milliseconds for smooth zooming.
 *  See ::OMX_PARAM_U32TYPE
 */
#define NVX_INDEX_CONFIG_SMOOTHZOOMTIME "OMX.Nvidia.index.config.smoothzoomtime"

/** Config extension index to abort zooming.
 *  See ::OMX_CONFIG_BOOLEANTYPE
 */
#define NVX_INDEX_CONFIG_ZOOMABORT "OMX.Nvidia.index.config.zoomabort"

/** Config extension index to set scale factor multiplier in digital zoom.
 *  See ::NVX_CONFIG_DZSCALEFACTORMULTIPLIER
 */
#define NVX_INDEX_CONFIG_DZSCALEFACTORMULTIPLIER "OMX.Nvidia.index.config.dzscalefactormultiplier"
/** Holds data to set scale factor multiplier in digital zoom. */
typedef struct NVX_CONFIG_DZSCALEFACTORMULTIPLIER
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_S32 ZoomFactorMultiplier;   /**< Scale factor from 100 to 800 (1.0-8.0) up to 8x digital */
}NVX_CONFIG_DZSCALEFACTORMULTIPLIER;

/** Config extension index to enable/disable camera preview.
 *  See ::OMX_CONFIG_BOOLEANTYPE
 */
#define NVX_INDEX_CONFIG_PREVIEWENABLE "OMX.Nvidia.index.config.previewenable"

/** Config extension index to enable/disable capture pause.
 *  See ::OMX_CONFIG_BOOLEANTYPE
 */
#define NVX_INDEX_CONFIG_CAPTUREPAUSE "OMX.Nvidia.index.config.capturepause"

/** Config extension index to setup converge and lock (half-press).
 * When this extension is called, the camera component will start
 * converging (achieving) auto focus, auto exposure, and/or auto
 * white balance. If the camera component achieves one or
 * more of these properties then it will lock settings of those
 * properties. If the camera component was unable to achieve one or
 * more of those properties in a certain time then it times out
 * for that property.
 *
 * The camera component notifies the application about
 * those properties that were achieved and those that timed out
 * using events. The following lists shows the different
 * events that are sent.
 * - Auto focus achieved (See ::NVX_EventCamera_AutoFocusAchieved)
 * - Auto focus timed out (See ::NVX_EventCamera_AutoFocusTimedOut)
 * - Auto exposure achieved (See ::NVX_EventCamera_AutoExposureAchieved)
 * - Auto exposure timed out (See ::NVX_EventCamera_AutoExposureTimedOut)
 * - Auto white balance achieved (See
 *      ::NVX_EventCamera_AutoWhiteBalanceAchieved)
 * - Auto white balance timed out (See
 *      ::NVX_EventCamera_AutoWhiteBalanceTimedOut)
 *
  * Although converge and lock is used with auto focus, auto exposure, and
 * auto white balance properties, an application can choose to
 * enable any one or any combination of these properties. When
 * any of these properties is disabled then half press will not
 * attempt to achieve that property.
 *
 *  See ::NVX_CONFIG_CONVERGEANDLOCK
 */
#define NVX_INDEX_CONFIG_CONVERGEANDLOCK "OMX.Nvidia.index.config.convergeandlock"
/** Holds data to setup converge and lock. */
typedef struct NVX_CONFIG_CONVERGEANDLOCK
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL bUnlock;               /**< Boolean to unlock AF, AE, AWB settings */
    OMX_BOOL bAutoFocus;            /**< Boolean to enable auto focus */
    OMX_BOOL bAutoExposure;         /**< Boolean to enable auto exposure */
    OMX_BOOL bAutoWhiteBalance;     /**< Boolean to enable auto white balance */
    OMX_U32  nTimeOutMS;            /**< Timeout in milliseconds */
} NVX_CONFIG_CONVERGEANDLOCK;

/** Config extension index to setup pre-capture converge.
 *  @deprecated This index is deprecated.
 */
#define NVX_INDEX_CONFIG_PRECAPTURECONVERGE "OMX.Nvidia.index.config.precaptureconverge"
/** Holds data to setup pre-capture converge. */
typedef struct NVX_CONFIG_PRECAPTURECONVERGE
{
    OMX_U32 nSize;                      /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;           /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;                 /**< Port that this struct applies to */
    OMX_BOOL bPrecaptureConverge;       /**< Boolean to enable pre-capture converge */
    OMX_BOOL bContinueDuringCapture;    /**< Boolean to enable continous converge during capture */
    OMX_U32  nTimeOutMS;                /**< Timeout in milliseconds */
} NVX_CONFIG_PRECAPTURECONVERGE;

/** Config extension index to setup min and max frame rate for camera.
 *  See ::NVX_CONFIG_AUTOFRAMERATE
 */
#define NVX_INDEX_CONFIG_AUTOFRAMERATE "OMX.Nvidia.index.config.autoframerate"
/** Holds data to setup auto frame rate for camera. */
typedef struct NVX_CONFIG_AUTOFRAMERATE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL bEnabled;              /**< Boolean to enable auto frame rate */
    OMX_S32  low;                   /**< Lowest frame rate allowed */
    OMX_S32  high;                  /**< Highest frame rate allowed */
} NVX_CONFIG_AUTOFRAMERATE;

/** Config extension index to specify exposure time range for camera.
 *
 * Sets the minimum and maximum exposure time or ISO sensitivty from which the
 * capture component's auto exposure picks values.
 *
 * The camera component supports auto exposure, where it
 * dynamically changes exposure time and sensor's sensitivty between a
 * minimum value and a maximum value specified
 * by config property data structure.
 * Minimum and maximum possible exposure times and ISO gains are dependent
 * on the camera sensor. Values above the maximum and below the minimum
 * supported by the camera sensor are clipped.
 *
 * Values are in units of milliseconds. To limit the exposure to nothing
 * slower than 1/30 second, a setting like {0, 33} would be used. 0 would
 * be replaced with the sensors fastest exposure limit by the driver.
 *
 * To limit the exposure to nothing faster than 1/30 second, set the
 * range to {33, 0}. The 0 for "high" would be replaced with the
 * sensor's slowest exposer limit.
 *
 *  See ::NVX_CONFIG_EXPOSURETIME_RANGE
 */
#define NVX_INDEX_CONFIG_EXPOSURETIMERANGE "OMX.Nvidia.index.config.exposuretimerange"
/** Holds data to specify exposure time range for camera. */
typedef struct NVX_CONFIG_EXPOSURETIME_RANGE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32  low;                   /**< Shortest exposure time allowed */
    OMX_U32  high;                  /**< Longest exposure time allowed */
} NVX_CONFIG_EXPOSURETIME_RANGE;

/** Config extension index to specify ISO sensitivity range for camera.
 *
 * Sets the minimum and maximum exposure time or ISO sensitivty from which the
 * capture component's auto exposure picks values.
 *
 * The camera component supports auto exposure, where it
 * dynamically changes exposure time and sensor's sensitivty between a
 * minimum value and a maximum value specified
 * by config property data structure.
 * Minimum and maximum possible exposure times and ISO gains are dependent
 * on the camera sensor. Values above the maximum and below the minimum
 * supported by the camera sensor are clipped.
 *
 * Values are in units of ISO. To limit the ISO gain to nothing more
 * than ISO400, a setting like {0, 400} would be used. 0 would be
 * replaced with the sensor's lowest supported gain limit by the
 * driver. To limit the ISO gain to nothing less than ISO400, a
 * setting like {400, 0} would be used. 0 would be replaced with the
 * sensor's highest supported gain limit by the driver. To reset to
 * initialization value, set to {0, 0}.
 *
 * Since the Auto exposure algorithm is 'exposure time priority', limiting
 * the ISO range will only dim the image if the ISO is limited by setting
 * the range. The exposure time is not recalculated based off of the
 * ISO range limit.
 *
 *  See ::NVX_CONFIG_ISOSENSITIVITY_RANGE
 */
#define NVX_INDEX_CONFIG_ISOSENSITIVITYRANGE "OMX.Nvidia.index.config.isosensitivityrange"
/** Holds data to specify ISO sensitivity range for camera. */
typedef struct NVX_CONFIG_ISOSENSITIVITY_RANGE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32  low;                   /**< Lowest ISO value allowed */
    OMX_U32  high;                  /**< Highest ISO value allowed */
} NVX_CONFIG_ISOSENSITIVITY_RANGE;

/** Config extension index to specify white balance correlated color
 * temperature (CCT) range for camera.
 *
 * Auto white balance will normally search a large set of illuminants to
 * estimate the CCT of a scene. This range can be limited to a subset
 * of the defined illuminants by setting this parameter.
 * The config property's low and high values in the range
 * specify the minimum and maximum CCT values to be searched. Legal values
 * are integers in units of degrees K. To select a specific illuminant,
 * the low and high members should be set to identical values. By default,
 * the low range value is 0, and the high value is MAX_INT.
 *
 *  See ::NVX_CONFIG_CCTWHITEBALANCE_RANGE
 */
#define NVX_INDEX_CONFIG_CCTWHITEBALANCERANGE "OMX.Nvidia.index.config.cctwhitebalancerange"
/** Holds data to specify whitebalance CCT range for camera. */
typedef struct NVX_CONFIG_CCTWHITEBALANCE_RANGE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32  low;                   /**< Low CCT range value */
    OMX_U32  high;                  /**< High CCT range value */
} NVX_CONFIG_CCTWHITEBALANCE_RANGE;

/** Config extension index to setup raw capture.
 *  See ::NVX_CONFIG_CAMERARAWCAPTURE
 */
#define NVX_INDEX_CONFIG_CAMERARAWCAPTURE "OMX.Nvidia.index.config.camera.rawcapture"
/** Holds data to setup raw capture. */
typedef struct NVX_CONFIG_CAMERARAWCAPTURE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32 nCaptureCount;          /**< Number of frames to capture */
    OMX_STRING Filename;            /**< File name to store into */
} NVX_CONFIG_CAMERARAWCAPTURE;

typedef enum
{
    NvxFlicker_Off,
    NvxFlicker_Auto,
    NvxFlicker_50HZ,
    NvxFlicker_60HZ
} ENvxFlickerType;

/** Config extension index to setup flicker.
 *  See ::NVX_CONFIG_FLICKER
 */
#define NVX_INDEX_CONFIG_FLICKER "OMX.Nvidia.index.config.flicker"
/** Holds data to setup flicker. */
typedef struct NVX_CONFIG_FLICKER
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    ENvxFlickerType eFlicker;       /**< Flicker type */
} NVX_CONFIG_FLICKER;

typedef enum NVX_IMAGEFILTERTYPE {
    NVX_ImageFilterPosterize = 0x30000000,
    NVX_ImageFilterSepia,
    NVX_ImageFilterBW,
    NVX_ImageFilterMax = 0x7FFFFFFF
} NVX_IMAGEFILTERTYPE;

/** Config extension index to setup hue.
 *  See ::NVX_CONFIG_HUETYPE
 */
#define NVX_INDEX_CONFIG_HUE "OMX.Nvidia.index.config.hue"
/** Holds data to setup hue. */
typedef struct NVX_CONFIG_HUETYPE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_S32 nHue;                   /**< Hue in degrees, 0 to 359 */
} NVX_CONFIG_HUETYPE;


/** Config extension index to control focus regions.
 * The rectangle coordinates are normalized to -1.0 to 1.0 on the
 * X and Y axis.
 * Auto focus region will work only when auto focus is enabled.
 *
 *  See ::NVX_CONFIG_FocusRegionsRect
 */
#define NVX_INDEX_CONFIG_FOCUSREGIONSRECT "OMX.Nvidia.index.config.focusregionsrect"
/** Holds data to control focus regions. */
typedef struct NVX_CONFIG_FocusRegionsRect
{
    OMX_U32      nRegions;                          /**< Number of regions */
    NVX_RectF32  regions[NVX_MAX_FOCUS_REGIONS];    /**< Array of NVX_RectF32 to specify each focus region */
} NVX_CONFIG_FocusRegionsRect;

/** Config extension index to control camera focusing regions sharpness values.
 *  You can use this interface to measure focus sharpness in manual mode.
 *  This interface fills in an ::NVX_CONFIG_FOCUSREGIONS_SHARPNESS structure.
 *  @note NVIDIA currently supports only one region, so the value of interest will be in:
 *  ::NVX_CONFIG_FOCUSREGIONS_SHARPNESS.nSharpness[0]
 */
#define NVX_INDEX_CONFIG_FOCUSREGIONSSHARPNESS "OMX.Nvidia.index.config.focusregionssharpness"
/** Holds data to control camera focusing regions sharpness values.
 * @sa NVX_INDEX_CONFIG_FOCUSREGIONSSHARPNESS
 */
typedef struct NVX_CONFIG_FOCUSREGIONS_SHARPNESS
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32           nRegions;                             /**< Number of currently active focusing regions */
    NVX_F32           nSharpness[NVX_MAX_FOCUS_REGIONS];    /**< Sharpness values for active focusing regions. @note Currently, NVIDIA supports only one region, so the value of interest will be in \c NVX_CONFIG_FOCUSREGIONS_SHARPNESS.nSharpness[0]. You can use the ::NVX_INDEX_CONFIG_FOCUSREGIONSSHARPNESS interface to measure focus sharpness in manual mode.  This interface fills in an ::NVX_CONFIG_FOCUSREGIONS_SHARPNESS structure.  */
} NVX_CONFIG_FOCUSREGIONS_SHARPNESS;

/** Config extension index for direct focuser control.
 *  @note This extension is deprecated. What you probably want is
 *  OMX_IndexConfigFocusControl.
 */
#define NVX_INDEX_CONFIG_DIRECTFOCUSERCONTROL "OMX.Nvidia.index.config.directfocusercontrol"
/** Holds a direct focuser control. */
typedef struct NVX_CONFIG_DIRECT_FOCUSERCONTROL
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32           nLength;                                              /**< Length of focuser control sequence */
    OMX_U32           nControl[NVX_MAX_DIRECT_FOCUSER_CONTROL_LENGTH];      /**< Focuser control sequence */
} NVX_CONFIG_DIRECT_FOCUSERCONTROL;

/** Config extension for querying camera focusing parameters.
 *
 *  See ::NVX_CONFIG_FOCUSER_PARAMETERS
 */
#define NVX_INDEX_CONFIG_FOCUSERPARAMETERS "OMX.Nvidia.index.config.focuserparameters"

typedef struct NVX_CONFIG_FOCUSER_PARAMETERS
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_S32           minPosition;          /**< Minimum focus position supported by Camera */
    OMX_S32           maxPosition;          /**< Maximum focus position supported by Camera */
    OMX_S32           hyperfocal;           /**< Hyperfocal focus position */
    OMX_S32           macro;                /**< Macro focus position */
    OMX_S32           powerSaving;          /**< Most power-efficient focus position */
                                            /**< (-1) if any position is good */
} NVX_CONFIG_FOCUSER_PARAMETERS;

/** Config extension for querying camera flash parameters.
 *
 *  See ::NVX_CONFIG_FLASH_PARAMETERS
 */
#define NVX_INDEX_CONFIG_FLASHPARAMETERS "OMX.Nvidia.index.config.flashparameters"

typedef struct NVX_CONFIG_FLASH_PARAMETERS
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL bPresent;              /**< Flash available for this sensor. */
} NVX_CONFIG_FLASH_PARAMETERS;

/** Config extension for querying camera low-level focuser device capabilities.
 *  See ::NVX_CONFIG_FOCUSER_CAPABILITIES
 */
#define NVX_INDEX_CONFIG_FOCUSERCAPABILITIES "OMX.Nvidia.index.config.focusercapabilities"
/** Holds a querying focuser capabilities. */
typedef struct NVX_CONFIG_FOCUSER_CAPABILITIES
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32           nRange;               /**< Range of focuser movement in device-dependent steps */
    NVX_F32           directionFactor;      /**< Scaling factor for focuser device movement in "infinity" and "macro" directions */
    OMX_U32           nSettleTime;          /**< Guaranteed settling time in millisecs */
} NVX_CONFIG_FOCUSER_CAPABILITIES;

/** Config extension to set the AF auto focusing speed. Fast is potentially
 *  less accurate; not fast is accurate but potentially quite slow.
 */
#define NVX_INDEX_CONFIG_AUTOFOCUS_SPEED "OMX.Nvidia.index.config.autofocusspeed"
typedef struct  {
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL bFast;                 /**< OMX_TRUE if you want fast focusing mode
                                         @note Slow mode may take several seconds
                                         to settle. */
} NVX_CONFIG_AUTOFOCUS_SPEED;

/** Holds a sensor mode configuration. */
typedef struct NVX_SENSORMODE
{
    OMX_S32 width;
    OMX_S32 height;
    NVX_F32 FrameRate;
} NVX_SENSORMODE;

/** Config extension index to control sensor mode list of supported resolution and frame rate.
 *  See ::NVX_CONFIG_SENSORMODELIST
 */
#define NVX_INDEX_CONFIG_SENSORMODELIST "OMX.Nvidia.index.config.sensormodelist"
/** Holds sensor modes. */
typedef struct NVX_CONFIG_SENSORMODELIST
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */

    OMX_U32 nSensorModes;                                   /**< Number of sensor modes */
    NVX_SENSORMODE SensorModeList[MAX_NUM_SENSOR_MODES];    /**< Array of sensor modes */
} NVX_CONFIG_SENSORMODELIST;

/** Config extension index to get sensor ID.
 *  See ::OMX_PARAM_U32TYPE
 */
#define NVX_INDEX_PARAM_SENSORID "OMX.Nvidia.index.config.sensorid"

/** Config/Param extension index to get number of available sensors on current platform.
 *   (Read-only)
 *  See ::OMX_PARAM_U32TYPE
 */
#define NVX_INDEX_PARAM_AVAILABLESENSORS "OMX.Nvidia.index.param.availablesensors"

/** Param extension index to set sensor mode.
 *  Sets the user-selected sensor mode (width, height, and frame rate).
 *  The default sensor mode is auto, i.e., \a WxHxFramerate is 0x0x0.
 *  The default auto mode (0x0x0) basically implies that the camera
 *  driver picks up the correct sensor mode using its own justification.
 *  Once the user-selected sensor mode is set using this interface,
 *  the default auto mode can be achieved again by setting 0x0x0 as
 *  sensor mode.
 *  See ::NVX_PARAM_SENSORMODE
 */
#define NVX_INDEX_PARAM_SENSORMODE "OMX.Nvidia.index.config.sensormode"

/** Holds the data for sensor mode setting. */
typedef struct NVX_PARAM_SENSORMODE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    NVX_SENSORMODE SensorMode;      /**< Sensor mode */
}NVX_PARAM_SENSORMODE;


/**
 * Defines the stereo camera modes.
 */
typedef enum {
    NvxStereo = 1,   /**< Sets the stereo camera to stereo mode. */
    NvxLeftOnly,     /**< Only the sensor on the left is on. */
    NvxRightOnly,     /**< Only the sensor on the right is on. */
    NvxStereoCameraForce32 = 0x7FFFFFFF
} ENvxCameraStereoCameraMode;

/** Param extension index to set camera mode, if the camera is stereo.
    Must be set before sensor power on.
    Has no effect for non-stereo cameras.
*/
#define NVX_INDEX_PARAM_STEREOCAMERAMODE "OMX.Nvidia.index.param.stereocameramode"
/** Holds the data for stereo mode setting. */
typedef struct NVX_PARAM_STEREOCAMERAMODE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    ENvxCameraStereoCameraMode StereoCameraMode;      /**< ENvxCameraStereoCameraMode to select stereo camera mode */
}NVX_PARAM_STEREOCAMERAMODE;

/** Config extension index to setup edge enhancement.
 * Edge enhancement increases the apparent sharpness of full
 * resolution images. It has no effect on downscaled images.
 *
 *  See ::NVX_CONFIG_EDGEENHANCEMENT
 */
#define NVX_INDEX_CONFIG_EDGEENHANCEMENT "OMX.Nvidia.index.config.edgeenhancement"
/** Holds data to setup edge enhancement. */
typedef struct NVX_CONFIG_EDGEENHANCEMENT
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */

    OMX_BOOL bEnable;               /**< Boolean to enable/disable edge enhancement */
    NVX_F32 strength;               /**< Strength of edge enhancement (float: 0 to 1) */
} NVX_CONFIG_EDGEENHANCEMENT;

/** Config extension index for ISP data dump.
 *  @deprecated This index is deprecated.
 */
#define NVX_INDEX_CONFIG_ISPDATA "OMX.Nvidia.index.config.ispdata"
/** @deprecated This structure is deprecated. */
typedef struct NVX_CONFIG_ISPDATA
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    NVX_F32 ilData;                 /**< ilData */
    NVX_F32 ilData2;                /**< ilData2 */
} NVX_CONFIG_ISPDATA;

/** Config extension index to set temporal tradeoff level for video encoder.
 *
 * For example:
 *  - 0 = Encoder delivers frames as fast as possible
 *  - 1 = Encoder drops 1 in 5 frames
 *  - 2 = Encoder drops 1 in 3 frames
 *  - 3 = Encoder drops 1 in 2 frames
 *  - 4 = Encoder drops 2 in 3 frames
 *
 *  See ::NVX_ENCODE_VIDEOTEMPORALTRADEOFF for values for video encode
 *       temporal tradeoff level.
 *
 *  @sa ::NVX_CONFIG_TEMPORALTRADEOFF
 */
#define NVX_INDEX_CONFIG_VIDEO_ENCODE_TEMPORALTRADEOFF "OMX.Nvidia.index.config.video.encode.temporaltradeoff"
/** Holds data to set temporal tradeoff. */
typedef struct NVX_CONFIG_TEMPORALTRADEOFF
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U32 TemporalTradeOffLevel;  /**< Temporal tradeoff level */
} NVX_CONFIG_TEMPORALTRADEOFF;

/** Defines the encode video temporal tradeoff types used to
  * set the video temporal tradeoff level.
  * @sa ::NVX_CONFIG_TEMPORALTRADEOFF 
  */
typedef enum NVX_ENCODE_VIDEOTEMPORALTRADEOFF{
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_DropNone= 0,
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_Drop1in5,
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_Drop1in3,
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_Drop1in2,
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_Drop2in3,
    NVX_ENCODE_VideoEncTemporalTradeoffLevel_Force32 = 0x7FFFFFFF
} NVX_ENCODE_VIDEOTEMPORALTRADEOFF;

/** Config extension index to get decoder configartion information from video encoder.
 *  See ::NVX_CONFIG_DecoderConfigInfo
 */
#define NVX_INDEX_CONFIG_VIDEO_ENCODE_DCI "OMX.Nvidia.index.config.video.encode.dci"
/** Holds data to get decoder configuration. */
typedef struct NVX_CONFIG_DecoderConfigInfo
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_U8 DecoderConfigInfo[NVX_VIDEOENC_DCI_SIZE];    /**< Decoder configuration info*/
    OMX_U8 SizeOfDecoderConfigInfo;                     /**< Size of decoder configuration info*/
}NVX_CONFIG_DecoderConfigInfo;


/** Config extension index for enabling per-frame user-space copies of the preview buffer.
 *
 *  See ::NVX_CONFIG_PREVIEW_FRAME_COPY
 */
#define NVX_INDEX_CONFIG_PREVIEW_FRAME_COPY "OMX.Nvidia.index.config.previewframecopy"
/** Holds data to enable/disable copying of preview frames */
typedef struct NVX_CONFIG_PREVIEW_FRAME_COPY
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL enable;                /**< enable/disable bit */
} NVX_CONFIG_PREVIEW_FRAME_COPY;

/** Config extension index for enabling per-still user-space copies of the
 *  still confirmation buffer.
 *
 *  See ::NVX_CONFIG_STILL_CONFIRMATION_FRAME_COPY
 */
#define NVX_INDEX_CONFIG_STILL_CONFIRMATION_FRAME_COPY "OMX.Nvidia.index.config.stillconfirmationframecopy"
/** Holds data to enable/disable copying of still-confirmation frames */
typedef struct NVX_CONFIG_STILL_CONFIRMATION_FRAME_COPY
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL enable;                /**< enable/disable bit */
} NVX_CONFIG_STILL_CONFIRMATION_FRAME_COPY;

/** Config extension index to set the camera configuration
 *
 * The ConfigString can be one of the below specified strings:
 *
 * - "ae.ConvergeSpeed=%f"
 * - "awb.ConvergeSpeed=%f"
 * - "ispFunction.lensShading=%s"
 *
 * where:
 * - %f is a float value
 * - %s is a string
 *
 * The float value for
 * - ae.ConvergeSpeed must be [0.01-1.0]
 * - awb.ConvergeSpeed must be [0.01-1.0]
 *
 * The string value for ispFunction.lensShading must be:
 * - TRUE/true to enable lens shading
 * - FALSE/false to disable lens shading
 * For ex : "ispFunction.lensShading=TRUE"
 *          "ispFunction.lensShading=FALSE"
 *
 * If the ConfigString passed does not "EXACTLY" match any of the above three strings, then
 * an error is returned.
 *
 * See ::NVX_CONFIG_CAMERACONFIGURATION
 */
#define NVX_INDEX_CONFIG_CAMERACONFIGURATION "OMX.Nvidia.index.config.cameraconfiguration"
/** Holds camera configuration string */
typedef struct NVX_CONFIG_CAMERACONFIGURATION
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_STRING ConfigString;        /**< Camera Configuration string */
} NVX_CONFIG_CAMERACONFIGURATION;

/** Config extension index for enabling per-still, user-space copies of the
 *  still YUV.
 *
 *  See ::NVX_CONFIG_STILL_YUV_FRAME_COPY
 */
#define NVX_INDEX_CONFIG_STILL_YUV_FRAME_COPY "OMX.Nvidia.index.config.stillYUVframecopy"
/** Holds data to enable/disable copying of still YUV frames. */
typedef struct NVX_CONFIG_STILL_YUV_FRAME_COPY
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL enable;                /**< Enable/disable bit */
} NVX_CONFIG_STILL_YUV_FRAME_COPY;

/** Config extension index to querying if the camera is stereo.
 */
#define NVX_INDEX_CONFIG_STEREOCAPABLE "OMX.Nvidia.index.config.stereocapable"
/** Holds the data from stereo capability query. */
typedef struct NVX_CONFIG_STEREOCAPABLE
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL StereoCapable;         /**< Boolean to indicate if this is a stereo camera */
} NVX_CONFIG_STEREOCAPABLE;

/** Config extension index for querying the scene brightness.
  * The brightness of the scene is the average scene luminance divided by the exposure value.
  * The higher the value, the brighter the scene.
  *
  * Read-only
 */
#define NVX_INDEX_CONFIG_SCENEBRIGHTNESS "OMX.Nvidia.index.config.scenebrightness"
/** Holds the data from scene brightness query. */
typedef struct NVX_CONFIG_SCENEBRIGHTNESS
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    NVX_F32 SceneBrightness;        /**< Scene brightness level */
} NVX_CONFIG_SCENEBRIGHTNESS;

/** Holds the custom block info from imager.  */
typedef struct NVX_CUSTOMINFO
{
  OMX_U32 SizeofData;
  OMX_STRING Data;
} NVX_CUSTOMINFO;

/** Config extension index to query custom block info from imager.
 */
#define NVX_INDEX_CONFIG_CUSTOMIZEDBLOCKINFO "OMX.Nvidia.index.config.customizedblockinfo"
/** Holds custom block info from imager. */
typedef struct NVX_CONFIG_CUSTOMIZEDBLOCKINFO
{
    OMX_U32 nSize;               /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;    /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;          /**< Port that this struct applies to */
    NVX_CUSTOMINFO CustomInfo;   /**< Custom info from imager */
} NVX_CONFIG_CUSTOMIZEDBLOCKINFO;

/** Config extension index for enabling advanced noise reduction.
 *
 *  See ::NVX_CONFIG_ADVANCED_NOISE_REDUCTION
 */
#define NVX_INDEX_CONFIG_ADVANCED_NOISE_REDUCTION "OMX.Nvidia.index.config.advancedNoiseReduction"
/** Holds data to control advanced noise reduction.

    The thresholds below are used to control the shape of the data
    Gaussian use to measure similarity between colors. Useful values
    will be between 0 and 0.2. The smaller the value, the more picky
    we are about considering something similar. The value of 0 means
    to disable filtering altogether for the corresponding channels.

    For each of luma, chroma:
      threshold[0] is used for horizontal filtering
      threshold[1] is used for vertical filtering
 */
typedef struct NVX_CONFIG_ADVANCED_NOISE_REDUCTION {
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL enable;                /**< Enable/disable bit */
    NVX_F32  lumaThreshold[2];      /**< Filter parameters for luma */
    NVX_F32  chromaThreshold[2];    /**< Filter parameters for chroma */
    OMX_U32  lumaIsoThreshold;      /**< Disable luma filtering if ISO below this thresh */
    OMX_U32  chromaIsoThreshold;    /**< Disable chroma filtering if ISO below this thresh */
} NVX_CONFIG_ADVANCED_NOISE_REDUCTION;

#define NVX_INDEX_CONFIG_BAYERGAINS "OMX.Nvidia.index.config.bayergains"
/** Reserved. Adjusts the per-channel bayer gains. */
typedef struct NVX_CONFIG_BAYERGAINS
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    NVX_F32 gains[4];                 /**< Gains per Bayer channel */
} NVX_CONFIG_BAYERGAINS;

#define NVX_INDEX_CONFIG_VIDEOFRAME_CONVERSION "OMX.Nvidia.index.config.videoframeconversion"
/** Reserved. Produces video frame data in user-specified format from OMX buffer. */

typedef enum NVX_VIDEOFRAME_FORMAT {
    NVX_VIDEOFRAME_FormatYUV420 = 0,
    NVX_VIDEOFRAME_FormatNV21,
    NVX_VIDEOFRAME_FormatInvalid = 0x7FFFFFFF
} NVX_VIDEOFRAME_FORMAT;

typedef struct NVX_CONFIG_VIDEOFRAME_CONVERSION
{
    OMX_U32 nSize;                    /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;         /**< NVX extensions specification version */
    OMX_U32 nPortIndex;               /**< Port that this struct applies to */
    NVX_VIDEOFRAME_FORMAT userFormat; /**< Requested YUV data format */
    OMX_PTR omxBuffer;                /**< OMX buffer to take data from */
    OMX_U32 nDataSize;                /**< Converted data size in bytes */
    OMX_PTR data;                     /**< Pointer to data */
} NVX_CONFIG_VIDEOFRAME_CONVERSION;

/** Config extension index for enabling capture priority.
 *  In capture priority mode, still/video capture has a higher priority than
 *  preview, so still/video frame rate may increase while preview frame rate
 *  may decrease.
 */
#define NVX_INDEX_CONFIG_CAPTURE_PRIORITY "OMX.Nvidia.index.config.capturepriority"
/** Holds data to enable/disable capture priority. */
typedef struct NVX_CONFIG_CAPTURE_PRIORITY
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    OMX_BOOL Enable;                /**< Enable/disable bit */
} NVX_CONFIG_CAPTURE_PRIORITY;

#define NVX_INDEX_CONFIG_LENS_PHYSICAL_ATTR "OMX.Nvidia.index.config.lensPhysicalAttr"
/** Config extension index for getting focal length. */
typedef struct NVX_CONFIG_LENS_PHYSICAL_ATTR
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    NVX_F32 eFocalLength;           /**< Focal length */
    NVX_F32 eHorizontalAngle;       /**< Horizontal view angle */
    NVX_F32 eVerticalAngle;         /**< Vertical view angle */
} NVX_CONFIG_LENS_PHYSICAL_ATTR;

/** Config extension index to get focus distances.
 *  Gets the distances from the camera to where an object appears to be
 *  in focus. The object is sharpest at the optimal focus distance. The
 *  depth of field is the far focus distance minus near focus distance.
 *  See ::NVX_CONFIG_FOCUSDISTANCES
 */
#define NVX_INDEX_CONFIG_FOCUSDISTANCES "OMX.Nvidia.index.config.focusdistances"
/** Holds data to setup focus distance. */
typedef struct NVX_CONFIG_FOCUSDISTANCES
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version
                                         information */
    OMX_U32 nPortIndex;             /**< Port that this struct applies to */
    NVX_F32 NearDistance;           /**< Near distance */
    NVX_F32 OptimalDistance;        /**< Optimal distance */
    NVX_F32 FarDistance;            /**< Far distance */
} NVX_CONFIG_FOCUSDISTANCES;

#define NVX_INDEX_CONFIG_MANUALTORCHAMOUNT  "OMX.Nvidia.index.config.manualtorchamount"
#define NVX_INDEX_CONFIG_MANUALFLASHAMOUNT  "OMX.Nvidia.index.config.manualflashamount"
#define NVX_INDEX_CONFIG_AUTOFLASHAMOUNT    "OMX.Nvidia.index.config.autoflashamount"

/** Holds data to specify the amount of manual torch, manual flash, or auto flash for camera. */
typedef struct NVX_CONFIG_FLASHTORCHAMOUNT
{
    OMX_U32 nSize;                  /**< Size of the structure in bytes */
    OMX_VERSIONTYPE nVersion;       /**< NVX extensions specification version information */
    NVX_F32  value;                 /**< Flash/torch value */
} NVX_CONFIG_FLASHTORCHAMOUNT;


#endif

/** @} */
/* File EOF */
