/*
 * Copyright (c) 2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */


#include "nvxframework.h"
#include "nvxgraph.h"
#include "NVOMX_IndexExtensions.h"

#define INVALID_PARAM               0x7FFFFFF
#define VIDEO_ENCODER_TYPE_H264    0
#define VIDEO_ENCODER_TYPE_MPEG4   1
#define VIDEO_ENCODER_TYPE_H263    2
#define MAX_OUTPUT_BUFFERS         10

typedef enum  {
    NvxOUTPUT_FORMAT_DEFAULT = 0,
    NvxOUTPUT_FORMAT_THREE_GPP = 1,
    /* These are audio only file formats */
    NvxOUTPUT_FORMAT_RAW_AMR = 2,
    NvxOUTPUT_FORMAT_MP4 = 3,
    NvxOUTPUT_FORMAT_Force32 = 0x7FFFFFF // must be last - used to validate format type
}NvxOutputFormat;

typedef enum
{
    NvxRecorderGraphState_Initialized = 0,
    NvxRecorderGraphState_DataSourceConfiguired,
    NvxRecorderGraphState_Prepared,
    NvxRecorderGraphState_Start,
    NvxRecorderGraphState_stop
} NvxRecorderGraphState;

typedef struct 
{
    void *pClientContext;
    NvxFillBufferDone pFillBufferDone;
    NvxEmptyBufferDone pEmptyBufferDone;
    NvxEventHandler pEventHandler;
}NvxRecorderAppContext;

typedef struct NvxRecorderParam_ {
    int nWidthCapture;
    int nHeightCapture;
    OMX_U32 nBitRate;
    OMX_U32 nFrameRate;
    OMX_U32 nAppType;
    OMX_U32 nQualityLevel;
    OMX_U32 nErrorResilLevel;
    OMX_U32 videoEncoderType;
    OMX_U32 videoEncodeLevel;
    OMX_U32 TemporalTradeOffLevel;
    OMX_S32  nRotation;
    OMX_MIRRORTYPE eMirror;
    OMX_U32 nInitialQPI;
    OMX_U32 nInitialQPP;
    NVX_VIDEO_RATECONTROL_MODE eRCMode;
    OMX_U32 nMinQpI;
    OMX_U32 nMaxQpI;
    OMX_U32 nMinQpP;
    OMX_U32 nMaxQpP;
    OMX_U32 nMinQpB;
    OMX_U32 nMaxQpB;
    OMX_U32 nSliceHeaderSpacing;
    OMX_U32 nIFrameInterval;
    OMX_BOOL bStringentBitrate;
} NvxRecorderParam;

typedef struct _NvxRecorderGraph
{
    NvxFramework        m_pOmxFramework;
    NvxGraph           *m_pGraph;
    NvxComponent       *m_pAudioCapturer;
    NvxComponent       *m_pAudioEncoder;
    NvxComponent       *m_pVideoEncoder;
    NvxComponent       *m_pAVWriter;
    NvxComponent       *m_phVideoEncoder;
    OMX_AUDIO_CODINGTYPE eAudioCodingType;
    OMX_AUDIO_AACPROFILETYPE eAacProfileType;
    char *outfile;
    OMX_U32 nAudioSampleRate;
    OMX_U32 nAudioBitRate;
    OMX_U32 nChannels;
    OMX_U32 eAudioSource;

    char *audioCapture;
    char *audioEnc;
    char *videoEnc;
    char *avWriter;
    OMX_BOOL   AudioEncEnable;
    OMX_BOOL   VideoEncEnable;
    OMX_BOOL   bUseNvBuffers;

    NvxRecorderGraphState eRecState;
    NvxRecorderAppContext RecordAppContext;
    OMX_BUFFERHEADERTYPE  *VideoEncoderInputBuffer[MAX_OUTPUT_BUFFERS];
    OMX_PARAM_PORTDEFINITIONTYPE oVideoEncInputPortDef;
    OMX_U32 nVideoInputBuffers;
    NvxRecorderParam RecorderParam;
    char *externalAudioSource;
    OMX_BOOL useExternalAudioSource;
    NvxOutputFormat eOutputFormat;
    OMX_U64 nFileSize;
    OMX_U64 nDuration;
} NvxRecorderGraph;

typedef struct _NvxRecorderAudioParams
{
    OMX_AUDIO_CODINGTYPE eAudioCodingType;
    OMX_AUDIO_AACPROFILETYPE eAacProfileType;
    OMX_U32 nAudioSampleRate;
    OMX_U32 nAudioBitRate;
    OMX_U32 nChannels;
    OMX_U32 eAudioSource;
}NvxRecorderAudioParams;

typedef struct _NvxRecorderVideoParams
{
    OMX_U32 nVideoCaptureWidth;
    OMX_U32 nVideoCaptureHeight;
    OMX_U32 nVideoFrameRate;
    OMX_BOOL bUseNvBuffers;
    OMX_U32 videoEncoderType;
    OMX_U32 nBitRate;
}NvxRecorderVideoParams;

OMX_ERRORTYPE NvxInitRecorderStruct(NvxRecorderGraph *pRecorder, 
                                    NvxFramework pOmxFramework);
                                    
void NvxRecorderGraphUseExternalAudioSource(NvxRecorderGraph *pRecorder, 
                                                    char *audRenderName);
                                    
OMX_ERRORTYPE NvxCreateRecorderGraph(NvxRecorderGraph *pGraph);

OMX_ERRORTYPE NvxChangeStateRecorderGraph(NvxRecorderGraph *m_pGraph, 
                                        NvxRecorderGraphState eRecGraphState);
                                    
OMX_ERRORTYPE NvxReleaseRecorderGraph(NvxRecorderGraph *pRecGraph);

void NvxRecorderRegisterClientCallbacks(NvxRecorderGraph *pGraph,
                                       void *pThis,
                                       NvxFillBufferDone pFillBufferDone,
                                       NvxEmptyBufferDone pEmptyBufferDone,
                                       NvxEventHandler pEventHandler);

OMX_ERRORTYPE NvxRecorderSetVideoParams(NvxRecorderGraph *pGraph, 
                                        NvxRecorderVideoParams *pParams);

OMX_ERRORTYPE NvxRecorderSetAudioParams(NvxRecorderGraph *pGraph, 
                                        NvxRecorderAudioParams *pParams);

OMX_ERRORTYPE NvxRecorderSetOutputFile(NvxRecorderGraph *pRecorder, 
                                       char *sPath);
OMX_ERRORTYPE NvxRecorderSetOutputFormat(NvxRecorderGraph *pRecorder,
                                            NvxOutputFormat Params);

OMX_BUFFERHEADERTYPE *NvxRecorderGetVideoEncoderBuffer(
                                        NvxRecorderGraph *pRecorder, 
                                        OMX_U32 index);

NvxComponent* NvxRecorderGetVideoEncoder(NvxRecorderGraph *pRecorder);

OMX_U32 NvxRecorderVideoEncoderGetBufferCount(NvxRecorderGraph *pRecorder);

OMX_ERRORTYPE NvxRecorderSetOutputFileSize(NvxRecorderGraph *pRecorder,
                                            OMX_U64 nSizeInBytes);

OMX_ERRORTYPE NvxRecorderSetMaxDuration(NvxRecorderGraph *pRecorder,
                                            OMX_U64 nDuration);
