#include <KD/kd.h>
#include "bubbleWindow.h"
#include "shaders.h"
#include "nvgl2demo_common.h"

// Set up bubble geometry
static void BubbleWindow_buildBubble(BubbleWindow *b, int depth)
{
    if (b->bubble) {
        Bubble_destroy(b->bubble);
    }
    switch (depth) {
        case 2: b->bubble = buildSphere( 6); break;
        case 3: b->bubble = buildSphere(13); break;
        case 4: b->bubble = buildSphere(25); break;
    }
    Bubble_calcNormals(b->bubble);
}

// Set up the bubble the bubble window
void NewBubbleWindow(BubbleWindow *b, int autopoke, float duration, int fpsFlag)
{
    b->time_frames = 30;
    b->frame_count = 0;
    b->startTime = 0.0;
    b->currentTime = 0.0;
    b->duration = duration;
    b->elapsed = 0;
    b->autopoke = autopoke;
    b->autopoke_framenum = 0;
    b->fps_flag = fpsFlag;
    b->swapInterval = 1;
    b->mouse_flag = true;
    b->one_frame = false;
    b->bubble = KD_NULL;
    b->env_cube = KD_NULL;
    b->window_width = 0;
    b->window_height = 0;
    b->mouse_x = 0;
    b->mouse_y = 0;
    b->screen_x = 0.0f;
    b->screen_y = 0.0f;
    b->mouse_down = false;
    b->near_height = 1.0f;
    b->near_distance = 2.0f;
    b->far_distance = 40.0f;
    b->eye_distance = 5.0f;
    b->mode = CUBE_MODE;
    b->heading = 0.0f;
    b->pitch = 0.0f;
    b->hv = 0.0f;
    b->pv = 0.0f;
    quat_identity(&b->orient);
    BubbleWindow_buildBubble(b, 4);

    b->nvtxf = KD_NULL;
}

// Free the bubble window
void DestroyBubbleWindow(BubbleWindow *b)
{
    if (b != KD_NULL) {
        if (b->env_cube != KD_NULL) EnvCube_destroy(b->env_cube);
        if (b->bubble != KD_NULL) Bubble_destroy(b->bubble);
        if (b->nvtxf != KD_NULL) nvtexfont2UnloadRasterFont(b->nvtxf);
    }
    FreeShaders();
}

///////////////////////////////////////////////////////////////////////////////

// Pick a point on the bubble to poke when mouse/space is pressed
static void BubbleWindow_pick(BubbleWindow *b)
{
    float4x4 m;
    float3 e;
    float3 n;
    float y = b->near_height * b->screen_y;
    float x = b->near_height * b->aspect * b->screen_x;

    e[0] = 0.0f;
    e[1] = 0.0f;
    e[2] = b->eye_distance;
    n[0] = x;
    n[1] = y;
    n[2] = -b->near_distance;
    quat_mat(m, &(b->orient));
    mat_invert(m);
    pnt_transform(e, m);
    vec_transform(n, m);
    Bubble_pick(b->bubble, e, n);
}

// Handle left mouse button press
void BubbleWindow_leftMouse(BubbleWindow *b, bool click)
{
    b->mouse_down = click;
    if (click == true) {
        BubbleWindow_pick(b);
    }
}

// Handle mouse movement
void BubbleWindow_mouse(BubbleWindow *b, int x, int y)
{
    int window_x = 0, window_y = 0;
    // Only update if within window boundaries
    if ((x > 0) && (x < b->window_width))
        b->mouse_x = x;
    if ((y > 0) && (y < b->window_height))
        b->mouse_y = y;
    // Map to [-1,+1]. Invert Y, because GL origin is lower left.
    window_y = b->window_height/2 - b->mouse_y;
    b->screen_y = ((float)window_y)/(((float)b->window_height)/2.0f);
    window_x = b->mouse_x - b->window_width/2;
    b->screen_x = ((float)window_x)/(((float)b->window_width)/2.0f);
    b->mouse_flag = true;
}

void BubbleWindow_callback(BubbleWindow *b, int key)
{
    int triCount = 0;
    int i = 0;

    switch (key) {
        case 'q':
        case 'Q':
            b->one_frame = true;
            break;
        case 'h':
        case 'H':
            NvGl2DemoLogMessage("\n"
                "h: help (this output)\n"
#ifdef USE_FAKE_MOUSE
                "i: move cursor up\n"
                "j: move cursor left\n"
                "k: move cursor down\n"
                "l: move cursor right\n"
#else
                "mouse: steer the bubble\n"
#endif
                "f: toggle fps display\n"
                "v: toggle wait for vsync (swap interval)\n"
                "m: toggle cursor display\n"
                "t: print triangles/frame\n"
                "p: render points\n"
                "w: render wireframe\n"
                "c: render full bubble\n"
#ifdef YOU_REALLY_WANT_UNSTABLE_GEOMETRY
                "2: low res bubble\n"
#endif
                "3: medium res bubble\n"
                "4: high res bubble\n"
#ifdef USE_FAKE_MOUSE
                "SPACE: poke the bubble\n"
#else
                "mouse button: poke the bubble\n"
#endif
                "q: quit\n"
                "\n");
            break;
#ifdef USE_FAKE_MOUSE
#define MOUSE_DELTA (10)
        case 'i':
        case 'I':
            BubbleWindow_mouse(b, b->mouse_x, (b->mouse_y)-MOUSE_DELTA);
            break;
        case 'k':
        case 'K':
            BubbleWindow_mouse(b, b->mouse_x, (b->mouse_y)+MOUSE_DELTA);
            break;
        case 'j':
        case 'J':
            BubbleWindow_mouse(b, (b->mouse_x)-MOUSE_DELTA, b->mouse_y);
            break;
        case 'l':
        case 'L':
            BubbleWindow_mouse(b, (b->mouse_x)+MOUSE_DELTA, b->mouse_y);
            break;
#endif
        case 'f':
        case 'F':
            b->fps_flag = !(b->fps_flag);
            break;
        // EGL 1.1
        case 'v':
        case 'V':
            {
                b->swapInterval = !(b->swapInterval);
                eglSwapInterval(gfxState.display, b->swapInterval);
                NvGl2DemoLogMessage("swap interval set to: %d\n", b->swapInterval);
            }
            break;
        case 'm':
        case 'M':
            b->mouse_flag = !(b->mouse_flag);
            break;
        case 't':
        case 'T':
            triCount = 0;
            // count up the triangles in the tristrips
            for ( i=0 ; i<(b->bubble)->numTristrips ; i++ ) {
                triCount += (b->bubble)->tristrips[i].numVerts - 2;
            }
            // add in the 6 walls of the cube that we're in
            triCount += 6*2;
            {
                NvGl2DemoLogMessage("%d triangles/frame\n", triCount);
            }
            break;
#ifdef YOU_REALLY_WANT_UNSTABLE_GEOMETRY
        case '2':
            BubbleWindow_buildBubble(b, 2);
            break;
#endif
        case '3':
            BubbleWindow_buildBubble(b, 3);
            break;
        case '4':
            BubbleWindow_buildBubble(b, 4);
            break;
        case 'p':
        case 'P':
            b->mode = POINT_MODE;
            break;
        case 'w':
        case 'W':
            b->mode = WIRE_MODE;
            break;
        case 'c':
        case 'C':
            b->mode = CUBE_MODE;
            break;
#ifdef USE_FAKE_MOUSE
        case ' ':
            BubbleWindow_leftMouse(b, true);
            BubbleWindow_leftMouse(b, false);
            break;
#endif
    }
}

///////////////////////////////////////////////////////////////////////////////

void BubbleWindow_init(BubbleWindow *b, GLsizei width, GLsizei height)
{
    // Load the shaders
    if (!LoadShaders()) return;

    // Set up the viewport
    BubbleWindow_reshapeViewport(b, width, height);

    // Set up the environment map
    b->mode = CUBE_MODE;
    b->env_cube = EnvCube_build();
    b->cube_texture = EnvCube_getCube(b->env_cube);

    // Initialize GL settings
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClearDepthf(1.0f);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    glUseProgram(prog_bubble);
    glUniform1i(uloc_bubbleTexUnit, 0);

    // Set up mouse
    BubbleWindow_mouse(b, (b->window_width)/3, (b->window_height)/2);

    //after porting nvtexfont
    b->nvtxf = nvtexfont2InitRasterFont(NV_TEXFONT_DEFAULT, 0, GL_TRUE,
                                     GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR);

    b->startTime = b->currentTime = (double)kdGetTimeUST()/ ((KDint64)1000*1000000);
}

// Draw the mouse pointer
static void BubbleWindow_drawmouse(BubbleWindow *b)
{
    const GLfloat crosshair[8] = { -3.5f,  0.0f,
                                    3.5f,  0.0f,
                                    0.0f, -3.5f,
                                    0.0f,  3.5f};

    // Draw crosshairs
    glUseProgram(prog_mouse);
    glUniform2f(uloc_mouseCenter, (GLfloat)b->mouse_x,
                                  (GLfloat)(b->window_height-1-b->mouse_y));
    glLineWidth(1.0f);
    glEnableVertexAttribArray(aloc_mouseVertex);
    glVertexAttribPointer(aloc_mouseVertex,
                          2, GL_FLOAT, GL_FALSE, 0, (void *)crosshair);
    glDrawArrays(GL_LINES, 0, 4);
    glDisableVertexAttribArray(aloc_mouseVertex);
}

// Draw the frame
static void BubbleWindow_draw(BubbleWindow *b)
{
    const float3 pitch_axis   = {1.0f, 0.0f, 0.0f};
    const float3 heading_axis = {0.0f, 1.0f, 0.0f};
    Quat     q1, q2;
    float4x4 modelview;
    float4x4 projection;
    float4x4 normal;
    float4x4 m;
    float3   v;

    // Clear the buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Figure out orientation and set up modelview to center bubble
    quat_setfv(&q1, b->pitch, pitch_axis);
    quat_setfv(&q2, b->heading, heading_axis);
    quat_multiply(&q1, &q2);
    quat_prescribe(&(b->orient), &q1);
    quat_mat(m, &(b->orient));
    mat_invert(m);
    v[0] =  0.0f;
    v[1] =  0.0f;
    v[2] = -1.0f;
    vec_transform(v, m);
    vec_scale(v, b->eye_distance);
    mat_identity(modelview);
    mat_translate(modelview, v[0], v[1], v[2]);

    // Compute the projection matrix for current orientation
    mat_identity(projection);
    mat_frustum(projection,
                -b->near_height * b->aspect,
                 b->near_height * b->aspect,
                -b->near_height,
                 b->near_height,
                 b->near_distance,
                 b->far_distance);
    quat_mat(m, &b->orient);
    mat_multiply(projection, m);

    // Load the environment cube program, set the matrix, and draw the cube
    glUseProgram(prog_cube);
    glUniformMatrix4fv(uloc_cubeProjMat, 1, GL_FALSE, (GLfloat*)&projection);
    EnvCube_draw(b->env_cube);

     // Perform bubble dynamics
    Bubble_calcVelocity(b->bubble);
    Bubble_filterVelocity(b->bubble);
    Bubble_calcNormals(b->bubble);

    // Draw the bubble. For full bubble we also need the normal matrix
    if (b->mode == CUBE_MODE) {
        glUseProgram(prog_bubble);
        kdMemcpy(normal, modelview, 16*sizeof(float));
        mat_transpose(normal);
        mat_invert_part(normal);
        glUniformMatrix4fv(uloc_bubbleProjMat, 1, GL_FALSE,
                           (GLfloat*)&projection);
        glUniformMatrix4fv(uloc_bubbleViewMat, 1, GL_FALSE,
                           (GLfloat*)&modelview);
        glUniformMatrix4fv(uloc_bubbleNormMat, 1, GL_FALSE,
                           (GLfloat*)&normal);
        Bubble_draw(b->bubble, b->env_cube->cube_texture);
    } else {
        glUseProgram(prog_mesh);
        glUniformMatrix4fv(uloc_meshProjMat, 1, GL_FALSE,
                           (GLfloat*)&projection);
        glUniformMatrix4fv(uloc_meshViewMat, 1, GL_FALSE,
                           (GLfloat*)&modelview);
        if (b->mode == WIRE_MODE)
            Bubble_drawEdges(b->bubble);
        else
            Bubble_drawVertices(b->bubble);
    }

    // Update fps
    if (b->fps_flag && b->elapsed != 0.0f) {
        float fps = b->time_frames/b->elapsed;
        KDchar numBuf[13];
        KDchar buf[30];
        kdFtostr(numBuf, 13, fps);
        kdStrcpy_s(buf, 30, "fps: ");
        kdStrncat_s(buf, 30, numBuf, 12);

        nvtexfont2RenderString_All(b->nvtxf, buf, 0.5f, -0.8f, 0.5f, 1.0f, 1.0f, 1.0f);

        if (b->frame_count == (b->time_frames - 1))
            NvGl2DemoLogMessage("fps: %f\n", fps);
    }

    // Draw mouse crosshairs
    if (b->mouse_flag) {
        glDisable(GL_DEPTH_TEST);
        BubbleWindow_drawmouse(b);
        glEnable(GL_DEPTH_TEST);
    }
}

// Process the next frame
void BubbleWindow_tick(BubbleWindow *b)
{
    // Update direction based on mouse position
    b->hv += b->screen_x * 0.001f;
    b->pv -= b->screen_y * 0.001f;
    b->pv -= b->pitch * 0.0006f;
    b->hv *= 0.94f;
    b->pv *= 0.94f;
    b->heading += b->hv;
    b->pitch   += b->pv;
    if (b->heading >  M_PI) b->heading -= (float)(2.0f*M_PI);
    if (b->heading < -M_PI) b->heading += (float)(2.0f*M_PI);

    // Draw the frame
    BubbleWindow_draw(b);

    // Update fps counters
    if (++(b->frame_count) == (b->time_frames)) {
        double time = (double)kdGetTimeUST() / ((KDint64)1000*1000000);
        b->frame_count = 0;
        b->elapsed = (float) (time - b->currentTime);
        b->currentTime = time;
    }

    // Check for exit
    if ((b->duration > 0.0) && (b->currentTime-b->startTime >= b->duration)) {
        b->one_frame = true;
    }

    // Handle auto-poking
    (b->autopoke_framenum)++;
    if (b->autopoke && b->autopoke_framenum == b->autopoke) {
        BubbleWindow_pick(b);
        b->autopoke_framenum = 0;
    }
}

// Initialize/change the viewport
void BubbleWindow_reshapeViewport(BubbleWindow *b, GLsizei w, GLsizei h)
{
    if (w==0 || h==0) return;
#ifdef USE_FAKE_MOUSE
    if (b->window_width && b->window_height) {
        b->mouse_x = (b->mouse_x * w) / b->window_width;
        b->mouse_y = (b->mouse_y * h) / b->window_height;
    }
#endif
    b->window_width = w;
    b->window_height = h;
    glViewport(0, 0, w, h);
    b->aspect = ((float)(b->window_width))/((float)(b->window_height));
    glUseProgram(prog_mouse);
    glUniform2f(uloc_mouseWindow, 1.0f/w, 1.0f/h);
}

