#include <KD/kd.h>
#include "envCube.h"
#include "shaders.h"
#include "nvgl2demo_common.h"


#define MAXSIZE 512

static char *textures[] = {
    "textures/right.ppm", "textures/left.ppm",
    "textures/top.ppm",   "textures/bottom.ppm",
    "textures/back.ppm", "textures/front.ppm"
};

static float cube_vertices[3*8] = {
    -1.0f,  1.0f, -1.0f, // l t f
    -1.0f, -1.0f, -1.0f, // l b f
     1.0f, -1.0f, -1.0f, // r b f
     1.0f,  1.0f, -1.0f, // r t f
    -1.0f,  1.0f,  1.0f, // l t b
    -1.0f, -1.0f,  1.0f, // l b b
     1.0f, -1.0f,  1.0f, // r b b
     1.0f,  1.0f,  1.0f  // r t b
};

static unsigned char cube_tristrip[] =
    {4, 7, 5, 6, 2, 7, 3, 4, 0, 5, 1, 2, 0, 3};

// gluBuild2DMipmaps() from mglu.c was used in original C++ code
// It's been slightly cut for OGLES2
static int gluBuild2DMipmaps(GLenum target, GLint internalFormat,
                             GLsizei width, GLsizei height,
                             GLenum format, GLenum type, const void *data)
{
    int lod, x, y, c;
    int pitch=0, shift, components;
    int comp[4];
    GLubyte *buf, *dst;
    const GLubyte *src1, *src2;

    // Handle some error cases
    if ((width < 1 || height < 1) ||                 // 0 width/height
        ((width&(width-1) || height&(height-1))) ||  // power of two check
        (type != GL_UNSIGNED_BYTE)) {                // must be UNSIGNED_BYTE
        return -1;
    }

    switch (format) {
        case GL_RGBA: components = 4; break;
        case GL_RGB: components = 3; break;
        case GL_LUMINANCE_ALPHA: components = 2; break;
        case GL_LUMINANCE:
        case GL_ALPHA: components = 1; break;
        default: components = 0; break;
    }

    // Reserve space for lod 1, (width/2) * (height/2) * 4, which is just...
    // For RGB or BGR, we reserve extra space.  So what?
    buf = kdMalloc(width*height);

    lod = 0;
    do {
        // Download a level
        glTexImage2D(target, lod, internalFormat, width, height, 0,
                     format, GL_UNSIGNED_BYTE, data);
        // After the 1x1, we're done.
        if (width == 1 && height == 1) {
            break;
        }

        // Observation:
        // Normal MxN downsampling will average a 2x2 block of texels into 1.
        // Nx1 or 1xN downsampling will average 2 successive texels into 1.
        // The Nx1 and 1xN cases can be handled without distinction between
        // width and height.
        dst = buf;
        src1 = src2 = data;
        // Nx1 or 1xN case
        if (width == 1 || height == 1) {
            shift = 1;
        }
        // General MxN case
        else {
            shift = 2;
            pitch = width*components;
            src2 += pitch;
        }

        if (width > 1)  width >>= 1;
        if (height > 1) height >>= 1;

        for (y=0; y<height; y++) {
            for (x=0; x<width; x++) {
                for (c=0; c<components; c++) {
                    comp[c] = *src1++;
                }
                for (c=0; c<components; c++) {
                    comp[c] += *src1++;
                }
                if (shift == 2) {
                    for (c=0; c<components; c++) {
                        comp[c] += *src2++;
                    }
                    for (c=0; c<components; c++) {
                        comp[c] += *src2++;
                    }
                }
                // Write the downsampled texel.
                for (c=0; c<components; c++) {
                    *dst++ = comp[c]>>shift;
                }
            }
            // For MxN case, skip a row.
            if (shift == 2) {
                src1 += pitch;
                src2 += pitch;
            }
        }

        data = buf;
        lod++;
    } while (1);

    kdFree(buf);
    return 0;
}

static unsigned int loadTextureCubemap(void)
{
    char path[1024];
    unsigned int id;
    unsigned int width, height;
    GLenum face;
    GLubyte *buf;
    KDFile *fp;
    char ppmheader[128];

    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_CUBE_MAP, id);
    glTexParameterf(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameterf(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameterf(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER,
                                                    GL_LINEAR_MIPMAP_LINEAR);

    for (face=GL_TEXTURE_CUBE_MAP_POSITIVE_X;
         face<=GL_TEXTURE_CUBE_MAP_NEGATIVE_Z; face++) {
        KDchar *endptr;
#       ifndef TEXTURE_PATH
#           define TEXTURE_PATH "/res/"
#       endif

        kdStrcpy_s(path, 1024, TEXTURE_PATH);
        kdStrncat_s(path, 1024, textures[face-GL_TEXTURE_CUBE_MAP_POSITIVE_X], 1024);

        buf = (GLubyte *) kdMalloc(MAXSIZE*MAXSIZE*3);

        fp = kdFopen(path, "r");
        if (!fp) {
            NvGl2DemoLogMessage("Failed to open the texture file: %s\n",textures[face-GL_TEXTURE_CUBE_MAP_POSITIVE_X]);
            kdExit(-1);
        }

        kdFread(ppmheader, 1, 128, fp);
        kdAssert(ppmheader[0] == 'P');
        kdAssert(ppmheader[1] == '6');

        width = kdStrtol(ppmheader + 2, &endptr, 10);
        height = kdStrtol(endptr, KD_NULL, 10);

        // IGL: sscanf on Win32 seems to like reading more than just the
        //      header. This is a crappy hack, but the header always seems
        //      to be 15 bytes long.
        kdFseek(fp, 15, KD_SEEK_SET);
        kdFread(buf, 1, width*height*3, fp);
        gluBuild2DMipmaps(face, GL_RGB, width, height,
                          GL_RGB, GL_UNSIGNED_BYTE, buf);
        kdFclose(fp);
        kdFree(buf);
    }


    glUseProgram(prog_cube);
    glUniform1i(uloc_cubeTexUnit, 0);

    return id;
}

EnvCube* EnvCube_build(void)
{
    EnvCube* e = kdMalloc(sizeof(EnvCube));
    if (e == KD_NULL) return KD_NULL;
    kdMemset(e, 0, sizeof(EnvCube));
    e->cube_texture = loadTextureCubemap();

    return e;
}

void EnvCube_draw(EnvCube* e)
{
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_CUBE_MAP, e->cube_texture);

    glEnableVertexAttribArray(aloc_cubeVertex);

    glVertexAttribPointer(aloc_cubeVertex,
                          3, GL_FLOAT, GL_FALSE, 0, cube_vertices);
    glDrawElements(GL_TRIANGLE_STRIP,
                   (sizeof(cube_tristrip) / sizeof(*cube_tristrip)),
                   GL_UNSIGNED_BYTE, cube_tristrip);

    glDisableVertexAttribArray(aloc_cubeVertex);
}

unsigned int EnvCube_getCube(EnvCube* e)
{
    return e->cube_texture;
}

void EnvCube_destroy(EnvCube* e)
{
    kdFree(e);
}
