#include <KD/kd.h>

#include "array.h"


////////////////////////////////////////////////////////////
// maintains an array of element (of same type) in a contiguous buffer.


// initial size of the array.
// We pick one that is large enough for the initial conditions and some
// growth.
static const int init_buff_size = 20000;

// how much the array buffer grows every time it exceeds the limit.
static const int buff_size_growth = 2;


// the constructor and destructor.
// elemsize is the byte size of the element to be stored in this array,
// i.e. sizeof(element type).
Array *Array_new(int elemsize)
{
    Array *o = (Array *)kdMalloc(sizeof(Array));
    Array_init(o, elemsize);
    return o;
}
void Array_delete(Array *o)
{
    Array_destroy(o);
    kdFree(o);
}


// besides constructor and destructor, this object has two more, which simply do
// not allocate and deallocate the object itself.
void Array_init(Array *o, int elemsize)
{
    o->elem_size = elemsize;
    o->elem_count = 0;
    o->buffer = KD_NULL;
    o->buff_size = 0;
}
void Array_destroy(Array *o)
{
    if(o->buffer) { kdFree(o->buffer); }
}


// set the size to zero, but keep the buffer as is.
void Array_clear(Array *o)
{
    o->elem_count = 0;
}


// access to the element specified by the index.
void *Array_get(Array *o, int i)
{
    kdAssert(i >= 0);
    kdAssert(i < o->elem_count);

    return (char *)o->buffer + i * o->elem_size;
}


// add an element at the end, extending the buffer space as needed.
void Array_push(Array *o, void *elem)
{
    // Make sure we have enough room in the buffer.
    if(o->buff_size < o->elem_size * (o->elem_count + 1))
    {
        // decide how much to allocate.
        if(o->buff_size == 0)
        {
            o->buff_size = o->elem_size * init_buff_size;
        }
        else
        {
            o->buff_size = o->buff_size * buff_size_growth;
        }

        o->buffer = kdRealloc(o->buffer, o->buff_size);
        kdAssert(o->buffer);
    }

    // write the element at the end location.
    kdMemcpy((char *)o->buffer + o->elem_count * o->elem_size, elem, o->elem_size);
    o->elem_count++;
}


// remove the last element, but do not shrink the buffer.
void Array_pop(Array *o)
{
    kdAssert(o->elem_count > 0);

    // simply ignore the last element.
    o->elem_count--;
}
